@@ -6,14 +6,15 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
-    from typing import Optional, Sequence, Tuple, Union, List
+    import numpy as np
+    from typing import Optional, Sequence, Tuple, Union
     from ._typing import ndarray, Device, Dtype, NestedSequence, SupportsBufferProtocol
 
 from typing import NamedTuple
 from types import ModuleType
 import inspect
 
-from ._helpers import _check_device, _is_numpy_array, array_namespace
+from ._helpers import _check_device, is_numpy_array, array_namespace
 
 # These functions are modified from the NumPy versions.
 
@@ -145,6 +146,9 @@ def zeros_like(
 
 # The functions here return namedtuples (np.unique() returns a normal
 # tuple).
+
+# Note that these named tuples aren't actually part of the standard namespace,
+# but I don't see any issue with exporting the names here regardless.
 class UniqueAllResult(NamedTuple):
     values: ndarray
     indices: ndarray
@@ -303,11 +307,13 @@ def _asarray(
         import numpy as xp
     elif namespace == 'cupy':
         import cupy as xp
+    elif namespace == 'dask.array':
+        import dask.array as xp
     else:
         raise ValueError("Unrecognized namespace argument to asarray()")
 
     _check_device(xp, device)
-    if _is_numpy_array(obj):
+    if is_numpy_array(obj):
         import numpy as np
         if hasattr(np, '_CopyMode'):
             # Not present in older NumPys
@@ -322,16 +328,32 @@ def _asarray(
     if copy in COPY_FALSE:
         # copy=False is not yet implemented in xp.asarray
         raise NotImplementedError("copy=False is not yet implemented")
-    if isinstance(obj, xp.ndarray):
+    if (hasattr(xp, "ndarray") and isinstance(obj, xp.ndarray)) or hasattr(obj, "__array__"):
+        #print('hit me')
         if dtype is not None and obj.dtype != dtype:
             copy = True
+        #print(copy)
         if copy in COPY_TRUE:
-            return xp.array(obj, copy=True, dtype=dtype)
+            copy_kwargs = {}
+            if namespace != "dask.array":
+                copy_kwargs["copy"] = True
+            else:
+                # No copy kw in dask.asarray so we go thorugh np.asarray first
+                # (like dask also does) but copy after
+                if dtype is None:
+                    # Same dtype copy is no-op in dask
+                    #print("in here?")
+                    return obj.copy()
+                import numpy as np
+                #print(obj)
+                obj = np.asarray(obj).copy()
+                #print(obj)
+            return xp.array(obj, dtype=dtype, **copy_kwargs)
         return obj
 
     return xp.asarray(obj, dtype=dtype, **kwargs)
 
-# xp.reshape calls the keyword argument 'newshape' instead of 'shape'
+# np.reshape calls the keyword argument 'newshape' instead of 'shape'
 def reshape(x: ndarray,
             /,
             shape: Tuple[int, ...],
@@ -340,8 +362,9 @@ def reshape(x: ndarray,
     if copy is True:
         x = x.copy()
     elif copy is False:
-        x.shape = shape
-        return x
+        y = x.view()
+        y.shape = shape
+        return y
     return xp.reshape(x, shape, **kwargs)
 
 # The descending keyword is new in sort and argsort, and 'kind' replaced with
@@ -385,6 +408,12 @@ def sort(
         res = xp.flip(res, axis=axis)
     return res
 
+# nonzero should error for zero-dimensional arrays
+def nonzero(x: ndarray, /, xp, **kwargs) -> Tuple[ndarray, ...]:
+    if x.ndim == 0:
+        raise ValueError("nonzero() does not support zero-dimensional arrays")
+    return xp.nonzero(x, **kwargs)
+
 # sum() and prod() should always upcast when dtype=None
 def sum(
     x: ndarray,
@@ -396,9 +425,12 @@ def sum(
     keepdims: bool = False,
     **kwargs,
 ) -> ndarray:
-    # `xp.sum` already upcasts integers, but not floats
-    if dtype is None and x.dtype == xp.float32:
-        dtype = xp.float64
+    # `xp.sum` already upcasts integers, but not floats or complexes
+    if dtype is None:
+        if x.dtype == xp.float32:
+            dtype = xp.float64
+        elif x.dtype == xp.complex64:
+            dtype = xp.complex128
     return xp.sum(x, axis=axis, dtype=dtype, keepdims=keepdims, **kwargs)
 
 def prod(
@@ -411,8 +443,11 @@ def prod(
     keepdims: bool = False,
     **kwargs,
 ) -> ndarray:
-    if dtype is None and x.dtype == xp.float32:
-        dtype = xp.float64
+    if dtype is None:
+        if x.dtype == xp.float32:
+            dtype = xp.float64
+        elif x.dtype == xp.complex64:
+            dtype = xp.complex128
     return xp.prod(x, dtype=dtype, axis=axis, keepdims=keepdims, **kwargs)
 
 # ceil, floor, and trunc return integers for integer inputs
@@ -454,10 +489,7 @@ def tensordot(x1: ndarray,
     return xp.tensordot(x1, x2, axes=axes, **kwargs)
 
 def vecdot(x1: ndarray, x2: ndarray, /, xp, *, axis: int = -1) -> ndarray:
-    ndim = max(x1.ndim, x2.ndim)
-    x1_shape = (1,)*(ndim - x1.ndim) + tuple(x1.shape)
-    x2_shape = (1,)*(ndim - x2.ndim) + tuple(x2.shape)
-    if x1_shape[axis] != x2_shape[axis]:
+    if x1.shape[axis] != x2.shape[axis]:
         raise ValueError("x1 and x2 must have the same size along the given axis")
 
     if hasattr(xp, 'broadcast_tensors'):
@@ -465,9 +497,9 @@ def vecdot(x1: ndarray, x2: ndarray, /, xp, *, axis: int = -1) -> ndarray:
     else:
         _broadcast = xp.broadcast_arrays
 
-    x1_, x2_ = _broadcast(x1, x2)
-    x1_ = xp.moveaxis(x1_, axis, -1)
-    x2_ = xp.moveaxis(x2_, axis, -1)
+    x1_ = xp.moveaxis(x1, axis, -1)
+    x2_ = xp.moveaxis(x2, axis, -1)
+    x1_, x2_ = _broadcast(x1_, x2_)
 
     res = x1_[..., None, :] @ x2_[..., None]
     return res[..., 0, 0]
@@ -511,7 +543,7 @@ def isdtype(
         # This will allow things that aren't required by the spec, like
         # isdtype(np.float64, float) or isdtype(np.int64, 'l'). Should we be
         # more strict here to match the type annotation? Note that the
-        # numpy.array_api implementation will be very strict.
+        # array_api_strict implementation will be very strict.
         return dtype == kind
 
 __all__ = ['arange', 'empty', 'empty_like', 'eye', 'full', 'full_like',
@@ -519,5 +551,5 @@ __all__ = ['arange', 'empty', 'empty_like', 'eye', 'full', 'full_like',
            'UniqueAllResult', 'UniqueCountsResult', 'UniqueInverseResult',
            'unique_all', 'unique_counts', 'unique_inverse', 'unique_values',
            'astype', 'std', 'var', 'permute_dims', 'reshape', 'argsort',
-           'sort', 'sum', 'prod', 'ceil', 'floor', 'trunc', 'matmul',
-           'matrix_transpose', 'tensordot', 'vecdot', 'isdtype']
+           'sort', 'nonzero', 'sum', 'prod', 'ceil', 'floor', 'trunc',
+           'matmul', 'matrix_transpose', 'tensordot', 'vecdot', 'isdtype']
